Programmed by Joao Pedro Neto.
Gif Pictures are Microsoft TradeMark.

/* Use:

<applet width="200" height="200" code="fsmEg.class">
  <param name="BACKGROUND" value="blutextb.gif">
  <param name="BGHEIGHT"   value="110">
  <param name="BGWIDTH"    value="116">
</applet>

 */


// ***************************************************************

import java.applet.*;
import java.awt.*;

public class fsmEg extends Applet {

  final int NIL        = -1;
  final int ST_PATROL  =  0; // Patrolling
  final int ST_FIGHT   =  1; // Fighting
  final int ST_HIDE    =  2; // Hiding
  final int ST_HEAL    =  3; // Healing
  final int ST_ESCAPE  =  4; // Escaping

  final int ST_DIM     =  5; // Number of States

  final int EV_ENEMY   =  0; // See enemy
  final int EV_NONE    =  1; // See no one
  final int EV_WOUNDED =  2; // It is wounded
  final int EV_FIND    =  3; // Found hiding place
  final int EV_HEALED  =  4; // It is healed

  final int EV_DIM     =  6; // Number of Events

  final int FSM[][] = new int[][] {

    // Define Finite State Machine (FSM):
    //  given an actual state and an actual event, the FSM
    //  informs to what state the agent must go.

    // EV_ENEMY  EV_NONE    EV_WOUNDED  EV_FIND   EV_HEALED

     { ST_FIGHT,   NIL    ,    NIL    ,   NIL  ,     NIL  }, // Patrol
     {    NIL  , ST_PATROL,  ST_HIDE  ,   NIL  ,     NIL  }, // Fighting
     {    NIL  , ST_HEAL  ,    NIL    , ST_HEAL, ST_FIGHT }, // Hiding
     { ST_HIDE ,   NIL    ,  ST_ESCAPE, ST_HEAL, ST_FIGHT }, // Healing
     {    NIL  , ST_HEAL  ,    NIL    , ST_HEAL, ST_FIGHT }, // Running

    }; //FSM

  String stateName[] = new String[] {

                    "Agent is Patrolling", 
                    "Agent is Fighting",
                    "Agent is Hiding",
                    "Agent is Healing",
                    "Agent is Escaping"
                  };

  int actualState;

  Button    buNextStep       = new Button("Next!");
  TextField tfCurrentState   = new TextField(15);

  CheckboxGroup cbgEvents    = new CheckboxGroup();
  Checkbox  cbNIL            = new Checkbox("No Event", true);
  Checkbox  cbEnemy          = new Checkbox("See Enemy", false);
  Checkbox  cbNone           = new Checkbox("See None", false);
  Checkbox  cbWound          = new Checkbox("Got Wounded", false);
  Checkbox  cbFound          = new Checkbox("Found Place to Hide", false);
  Checkbox  cbHealed         = new Checkbox("Wounds Healed", false);

  Image gifBackground, gifStates[] = new Image[ST_DIM];
  int   bgWidth, bgHeight;

  Panel eastPanel  = new Panel();
  Panel southPanel = new Panel();

  public void init() {

    Integer aux = new Integer(0);

    // get paramenters for background
    String arg = getParameter("BACKGROUND");
    gifBackground = getImage(getCodeBase(), arg );

    arg = getParameter("BGWIDTH");
    aux = Integer.valueOf(arg);
    bgWidth = aux.intValue();

    arg = getParameter("BGHEIGHT");
    aux = Integer.valueOf(arg);
    bgHeight = aux.intValue();

    // init states gifs

    gifStates[0] = getImage(getCodeBase(), "fsmPatrol.gif" );
    gifStates[1] = getImage(getCodeBase(), "fsmFight.gif" );
    gifStates[2] = getImage(getCodeBase(), "fsmHide.gif" );
    gifStates[3] = getImage(getCodeBase(), "fsmHeal.gif" );
    gifStates[4] = getImage(getCodeBase(), "fsmEscape.gif" );

    // init agent state

    actualState = ST_PATROL;

    // init components

    setLayout(new BorderLayout());

    tfCurrentState.setBackground(Color.white);

    tfCurrentState.setText(stateName[actualState]);
    tfCurrentState.setEditable(false);
    tfCurrentState.setBackground(Color.white);

    cbNIL.setBackground(Color.white);
    cbEnemy.setBackground(Color.white);
    cbNone.setBackground(Color.white);
    cbWound.setBackground(Color.white);
    cbFound.setBackground(Color.white);
    cbHealed.setBackground(Color.white);

    cbNIL.setCheckboxGroup(cbgEvents);
    cbEnemy.setCheckboxGroup(cbgEvents);
    cbNone.setCheckboxGroup(cbgEvents);
    cbWound.setCheckboxGroup(cbgEvents);
    cbFound.setCheckboxGroup(cbgEvents);
    cbHealed.setCheckboxGroup(cbgEvents);

    eastPanel.setLayout(new GridLayout(EV_DIM,1,0,-16));
    eastPanel.add(cbNIL);
    eastPanel.add(cbEnemy);
    eastPanel.add(cbNone);
    eastPanel.add(cbWound);
    eastPanel.add(cbFound);
    eastPanel.add(cbHealed);
    eastPanel.setBackground(Color.white);
    add("East", eastPanel);

    southPanel.add(tfCurrentState);
    southPanel.add(buNextStep);
    southPanel.setBackground(Color.white);
    add("South", southPanel);


  } //init()

  // ************************************************

  public boolean action (Event evt, Object arg) {


     if (evt.target instanceof Button) {  // deal with buttons

        if (arg.equals("Next!")) {

           computeNextStep();
           repaint();
           return true;

        }

     } //if(button)

     return false;

  } //action()

  // ************************************************

  public void paint( Graphics g ) {

    // drawBackGround

    for (int y=0; y<getSize().height; y+=bgHeight)
      for (int x=0; x<getSize().width; x+=bgHeight)
        g.drawImage(gifBackground,x,y,this);

    // draw state image

    g.drawImage(gifStates[actualState],10,55,this);

  } //paint()


  // ************************************************

  public void computeNextStep() {

    int nextState = NIL;

    if (cbEnemy.getState()) {
      nextState = FSM[actualState][0];
      cbEnemy.setState(false);
    }
    else if (cbNone.getState()) {
      nextState = FSM[actualState][1];
      cbNone.setState(false);
    }
    else if (cbWound.getState()) { 
      nextState = FSM[actualState][2];
      cbWound.setState(false);
    }
    else if (cbFound.getState()) { 
      nextState = FSM[actualState][3];
      cbFound.setState(false);
    }
    else if (cbHealed.getState()) { 
      nextState = FSM[actualState][4];
      cbHealed.setState(false);
    }

    if (nextState!=NIL)
      actualState = nextState;

    cbgEvents.setSelectedCheckbox(cbNIL);

    tfCurrentState.setText(stateName[actualState]);
    
  } //computeNextStep;

} // class fsmEg

// ***************************************************************

